;===============================================================================
;
; fair damage 1.0
;
; - changes the default damage algorithm for a new one.
;
;===============================================================================

header
hirom

;===============================================================================
;
; null vigor multiplication
;
;===============================================================================

org     $C22890
        NOP                      ;nothing
        NOP                      ;nothing
warnpc  $C22892

;===============================================================================
;
; redirect damage calculation for spell usage in menu
;
;===============================================================================

org     $C24775
        JMP damage_default
warnpc  $C24778

;===============================================================================
;
; auxiliary functions
;
;===============================================================================
org     $C22B69

;-------------------------------------------------------------------------------
; magic damage redirection
;-------------------------------------------------------------------------------

    JMP main

;-------------------------------------------------------------------------------
; increment damage by 1.5
;-------------------------------------------------------------------------------
damage_plus:

    ;init
        REP #$20                    ;mode 2A

    ;damage = damage + (damage/2)
        LDA $11B0                   ;damage
        LSR                         ;damage = damage / 2
        ADC $11B0                   ;A = damage + (damage/2)

    ;check and adjust damage
        BCS .damage_cap             ;cap damage if overflow
        CMP #$270F                  ;9999 damage
        BCC .damage_save            ;save if damage < 9999

    ;cap damage
.damage_cap
        LDA #$270F                  ;9999 damage
.damage_save
        STA $11B0                   ;save damage

    ;exit
        SEP #$20                    ;mode 1A
        RTS                         ;exit

;-------------------------------------------------------------------------------
; decrement damage
;-------------------------------------------------------------------------------
damage_minus:

    ;init
        REP #$20                    ;mode 2A

    ;damage = damage/2
        LDA $11B0                   ;damage
        LSR                         ;damage = damage/2
        STA $11B0                   ;save it

    ;exit
        SEP #$20                    ;mode 1A
        RTS                         ;exit

;-------------------------------------------------------------------------------
; null damage
;-------------------------------------------------------------------------------
damage_null:

    STZ $11B0                       ;null low byte
    STZ $11B1                       ;null high byte
    RTS                             ;exit

warnpc  $C22B9D

;===============================================================================
;
; main damage algorithm
;
;===============================================================================
org     $C22B9D

;-------------------------------------------------------------------------------
; main algorithm
;-------------------------------------------------------------------------------
main:

        JSR damage_default          ;setup default damage
        JSR damage_weapon           ;setup weapon damage
        RTS                         ;exit

;-------------------------------------------------------------------------------
; default damage
;-------------------------------------------------------------------------------
damage_default:

    ;check special cases
        LDA $11AF                   ;level
        BNE .skip1                  ;skip if level isn't zero

    ;check runic
        LDA $11AE                   ;vigor or magic
        BNE .skip1                  ;skip if vigor or magic isn't zero

    ;setup runic/item case
        LDA $11A6                   ;spell power
        STA $11B0                   ;save damage (low byte)
        STZ $11B1                   ;save damage (high byte)
        RTS

.skip1
    ;check null damage
        LDA $11AE                   ;vigor or magic power
        BEQ damage_null             ;null damage if it is zero
        LDA $11A6                   ;weapon or spell power
        BEQ damage_null             ;null damage if it is zero

    ;setup average factor
        LDA $11AE                   ;vigor or magic power
        ADC $11A6                   ;weapon or spell power
        ROR                         ;make average from two
        STA $E8                     ;M1 variable: average from above
        STA $11B0                   ;setup minimum damage (low byte)
        STZ $11B1                   ;null minimum damage (high byte)

    ;setup level factor
        LDA $11AF                   ;level
        ASL                         ;M2 variable: level * 2
        PHA                         ;save it
        XBA
        PLA                         ;M3 variable: level * 2

    ;multiply
        JSR $4781                   ;multiply A and B: M2 * M3
        JSR $47B7                   ;multiply above by M1. Result = M1 * M2 * M3 / 256

    ;save damage
        REP #$20                    ;mode 2A
        ADC $11B0                   ;add with minimum damage
        BCS .cap_set
        CMP #$270F                  ;9999 damage
        BCC .cap_skip
.cap_set
        LDA #$270F                  ;maximum damage (9999)
.cap_skip
        STA $11B0                   ;update damage
        SEP #$20                    ;mode 1A

    ;exit
        RTS

;-------------------------------------------------------------------------------
; adjust weapon damage
;-------------------------------------------------------------------------------
damage_weapon:

    ;check fight command
        LDA $B5                     ;command
        BNE .exit                   ;exit if not fight

    ;check gauntlet
        LDA $B3                     ;flags byte
        BIT #$40                    ;gauntlet bit
        BNE .skip_gauntlet          ;skip if not

    ;setup gauntlet case
        JSR damage_plus             ;damage = damage * 1.5
.skip_gauntlet

    ;check offering
        LDA $3C58,X                 ;flags byte
        BIT #$01                    ;offering bit
        BEQ .skip_offering          ;skip if not

    ;setup offering case
        JSR damage_minus            ;damage = damage * 0.5
.skip_offering

    ;check genji glove
        LDA $11CF                   ;flags byte
        BIT #$10                    ;genji glove
        BEQ .skip_genji

    ;setup genji glove
        JSR damage_minus            ;damage = damage * 0.5
        JSR damage_plus             ;damage = damage * 1.5

.skip_genji
    ;exit
.exit
    RTS

warnpc  $C22C21
